# (c) Copyright 2014,2015 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import unittest

from freezer.storage import ssh

class TestSshStorage(unittest.TestCase):

    def test_constructor(self):
        ssh.SshStorage.init = lambda x: x
        ssh.SshStorage(
            "test_dir", "test_work_dir", "test_key", "test_name", "test_ip",
            "teset_port")

    def test_info(self):
        ssh.SshStorage.init = lambda x: x
        ssh.SshStorage(
            "test_dir", "test_work_dir", "test_key", "test_name", "test_ip",
            "teset_port").info()
